package com.yidianling.muse.activity

import android.content.Intent
import android.media.MediaPlayer
import android.os.Bundle
import android.view.View
import android.widget.SeekBar
import com.alibaba.android.arouter.facade.annotation.Route
import com.bumptech.glide.Glide
import com.google.gson.Gson
import com.ydl.ydlcommon.base.BaseActivity
import com.ydl.ydlcommon.bean.StatusBarOptions
import com.ydl.ydlcommon.data.http.RxUtils
import com.ydl.ydlcommon.data.http.ThrowableConsumer
import com.ydl.ydlcommon.utils.StatusBarUtils.Companion.setTransparentForImageView
import com.ydl.ydlcommon.utils.StatusBarUtils.Companion.statusBarLightMode
import com.ydl.ydlcommon.utils.remind.ToastHelper
import com.yidianling.home.http.MuseHttp
import com.yidianling.muse.R
import com.yidianling.muse.helper.MediaPlayerManager
import com.yidianling.muse.utils.MediaPlayerTimeUtil
import com.yidianling.muse.widget.QuitMeditationDialog
import com.yidianling.muse.widget.ShareMeditationDialog
import io.reactivex.Observable
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.activity_play_meditation.*
import kotlinx.android.synthetic.main.player_control_view.*
import java.util.*
import java.util.concurrent.TimeUnit


@Route(path = "/muse/play")
class PlayMeditationActivity : BaseActivity() {

    private var meditationId = 0L
    private var meditationType = 0

    private var bgUrl =
        "https://gimg2.baidu.com/image_search/src=http%3A%2F%2Fcuimianxinli.com%2Fupload%2F2016-07%2F16072613412794.jpg&refer=http%3A%2F%2Fcuimianxinli.com&app=2002&size=f9999,10000&q=a80&n=0&g=0n&fmt=auto?sec=1654150276&t=393628420414c371b4b00540943cf0c7"

    private val path =
        "http://file.kuyinyun.com/group1/M00/90/B7/rBBGdFPXJNeAM-nhABeMElAM6bY151.mp3"

    private val newPath =
        "http://file.diyring.cc/UserRingWorksFile/0/50284096.mp3"

    private var collected = false
    private var isPlaying = false

    private var shareDialog: ShareMeditationDialog? = null
    private var quitDialog: QuitMeditationDialog? = null

    private var mTimer = Timer()
    private var mMediaPlayer: MediaPlayer? = null

    private var duration = 0

    private var isSeekbarChanging = false

    private var mDisposable: Disposable? = null
    private var mObservable: Observable<Long>? = null
    private var mObserver: Observer<Long>? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        setTransparentForImageView(this, null)
        statusBarLightMode(this)
        super.onCreate(savedInstanceState)

        meditationId = intent?.getLongExtra("MEDITATION_ID", 0L) ?: 0L
        meditationType = intent?.getIntExtra("MEDITATION_TYPE", 0) ?: 0

        initView()

        getData()

    }

    override fun initDataAndEvent() {

    }


    private fun getData() {

        updateUIByMeditationType()

//        声音详情
        if (meditationType == 0) {
            MuseHttp.getInstance().getPureMusicPlayDetail(
                meditionType = meditationType,
                meditationId = meditationId
            )
                .compose(RxUtils.applySchedulers())
                .map { it }
                .subscribe(Consumer {

                    if (it.code.equals("200") && it.data != null) {

                        val module = it.data

                        val title = module.title ?: "未知标题"

                        val desc = module.dec ?: "写着人们心疼的曲折，我想我很快乐，做一个旁观者"

                        val bgImageUrl =
                            if (module.mditationBackgroundImage.isNullOrEmpty()) {
                                bgUrl
                            } else {
                                module.mditationBackgroundImage
                            }

                        val mediaUrl = module.mediaUrl

                        val currentPosition = module.broadcastTime

                        Glide.with(this)
                            .load(bgImageUrl)
                            .into(iv_bg)


                        tv_title.text = title
                        tv_content.text = desc

                        initMediaPlayer(mediaUrl)


                    }

                }, object : ThrowableConsumer() {
                    override fun accept(msg: String) {

                    }
                })
        } else if (meditationType == 1) {
            MuseHttp.getInstance().getMeditationPlayDetail(
                meditionType = 1,
                mediaId = 1, meditationId = 1
            )
                .compose(RxUtils.applySchedulers())
                .map { it }
                .filter { true }
                .subscribe(Consumer {

                    if (it.code.equals("200") && it.data != null) {

                        val module = it.data

                        val title = module.title ?: "未知标题"

                        val desc = module.dec ?: "写着人们心疼的曲折，我想我很快乐，做一个旁观者"

                        val bgImageUrl = module.mditationBackgroundImage ?: ""

                        val mediaUrl = module.mediaUrl

                        val currentPosition = module.broadcastTime

                        Glide.with(this)
                            .load(bgImageUrl)
                            .into(iv_bg)


                        tv_title.text = title
                        tv_content.text = desc


                    }

                    var dataJson = Gson().toJson(it.data)


                }, object : ThrowableConsumer() {
                    override fun accept(msg: String) {

                    }
                })
        }


    }

    private fun updateUIByMeditationType() {
//        声音类型
        if (meditationType == 0) {
            exo_rew.visibility = View.GONE
            exo_ffwd.visibility = View.GONE
            exo_prev.visibility = View.GONE
            exo_next.visibility = View.GONE
            iv_time_off.visibility = View.GONE
            tv_time_off.visibility = View.GONE

            ll_pure_music_time_off.visibility = View.VISIBLE
            iv_pure_music_list.visibility = View.VISIBLE

        } else {
            exo_rew.visibility = View.VISIBLE
            exo_ffwd.visibility = View.VISIBLE
            exo_prev.visibility = View.VISIBLE
            exo_next.visibility = View.VISIBLE
            iv_time_off.visibility = View.VISIBLE
            tv_time_off.visibility = View.VISIBLE

            ll_pure_music_time_off.visibility = View.GONE
            iv_pure_music_list.visibility = View.GONE
        }

        initClickListener()

    }

    private fun initClickListener() {
        iv_pure_music_list.setOnClickListener {

        }

        iv_collect.setOnClickListener {
            collected = !collected
            if (collected) {
                ToastHelper.show("收藏成功！")
                iv_collect.setImageResource(R.drawable.icon_meditation_collected)
            } else {
                ToastHelper.show("取消收藏！")
                iv_collect.setImageResource(R.drawable.icon_play_meditation_collect)
            }
        }

    }

    override fun layoutResId(): Int {
        return R.layout.activity_play_meditation
    }

    override fun getStatusViewOptions(): StatusBarOptions {
        return StatusBarOptions(true, false)
    }

    private fun initView() {

        iv_close.setOnClickListener {
            if (quitDialog == null) {
                quitDialog = QuitMeditationDialog.newInstance()
            }
            if (quitDialog != null && quitDialog?.isAdded == false) {
                quitDialog?.setListener(object : QuitMeditationDialog.ClickListener {
                    override fun quit() {
                        ToastHelper.show("结束练习")
                        finish()
                    }
                })
                quitDialog?.show(supportFragmentManager, QuitMeditationDialog.TAG)
            }
        }

        iv_back.setOnClickListener {
            finish()
        }

        iv_share.setOnClickListener {
            if (shareDialog == null) {
                shareDialog = ShareMeditationDialog
                    .newInstance(
                        shareImageUrl = bgUrl,
                        shareContent = "老铁，我是你要分享的内容"
                    )
            }
            if (shareDialog != null && shareDialog?.isAdded == false) {
                shareDialog?.show(supportFragmentManager, ShareMeditationDialog.TAG)
            }
        }

        iv_time_off.setOnClickListener {
            startActivityForResult(
                Intent(this, MeditationTimeOffActivity::class.java),
                request_code
            )
        }

        ll_pure_music_time_off.setOnClickListener {
            startActivityForResult(
                Intent(this, MeditationTimeOffActivity::class.java),
                request_code
            )
        }

        exo_play.setOnClickListener {
            if (isPlaying) {
                MediaPlayerManager.getInstance(this).pause()
            } else {
                val duration = mMediaPlayer?.duration

                if (MediaPlayerManager.getInstance(this).path.isNullOrBlank()) {
                    MediaPlayerManager.getInstance(this).path = path

                } else {
                    MediaPlayerManager.getInstance(this).play()
                }
                if (duration != null) {
                    seekbar_play_progress.max = duration
                }
                mTimer.schedule(object : TimerTask() {
                    override fun run() {
                        if (!isSeekbarChanging) {
                            seekbar_play_progress.progress = mMediaPlayer?.currentPosition ?: 0
                        }
                    }
                }, 0, 50)

            }
            isPlaying = !isPlaying
            if (isPlaying) {
                exo_play.setImageResource(R.drawable.icon_pause_play)
            } else {
                exo_play.setImageResource(R.drawable.icon_pause_meditation)
            }

        }

        exo_prev.setOnClickListener {
            MediaPlayerManager.getInstance(this).setNewPath(newPath)
            isPlaying = if (mMediaPlayer?.isPlaying == true) {
                exo_play.setImageResource(R.drawable.icon_pause_play)
                if (duration != null) {
                    seekbar_play_progress.max = duration
                }
                mTimer.schedule(object : TimerTask() {
                    override fun run() {
                        seekbar_play_progress.progress = mMediaPlayer?.currentPosition ?: 0
                    }
                }, 0, 50)
                true
            } else {
                exo_play.setImageResource(R.drawable.icon_pause_meditation)
                false
            }
        }

        exo_next.setOnClickListener {
            MediaPlayerManager.getInstance(this).setNewPath(newPath)
            isPlaying = if (mMediaPlayer?.isPlaying == true) {
                exo_play.setImageResource(R.drawable.icon_pause_play)
                if (duration != null) {
                    seekbar_play_progress.max = duration
                }
                mTimer.schedule(object : TimerTask() {
                    override fun run() {
                        if (!isSeekbarChanging) {
                            seekbar_play_progress.progress = mMediaPlayer?.currentPosition ?: 0
                        }
                    }
                }, 0, 50)
                true
            } else {
                exo_play.setImageResource(R.drawable.icon_pause_meditation)
                false
            }
        }

        exo_rew.setOnClickListener {
            if (mMediaPlayer != null) {
                val currentPosition = mMediaPlayer!!.currentPosition
                val seekPosition = 0.coerceAtLeast(currentPosition - 15000)
                mMediaPlayer!!.seekTo(seekPosition)
            }
        }

        exo_ffwd.setOnClickListener {
            if (mMediaPlayer != null) {
                val currentPosition = mMediaPlayer!!.currentPosition
                val seekPosition = duration.coerceAtMost(currentPosition + 15000)
                mMediaPlayer!!.seekTo(seekPosition)
            }
        }

        seekbar_play_progress.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                if (mMediaPlayer != null) {
                    val totalDuration = mMediaPlayer!!.duration / 1000
                    val position = mMediaPlayer!!.currentPosition
                    exo_position.text = MediaPlayerTimeUtil.calculateTime(position / 1000)
                    exo_duration.text = MediaPlayerTimeUtil.calculateTime(totalDuration)
                }

            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {
                isSeekbarChanging = true
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
                isSeekbarChanging = false
                seekBar?.progress?.let {
                    MediaPlayerManager.getInstance(this@PlayMeditationActivity).seekTo(
                        it
                    )
                    if (mMediaPlayer != null) {
                        exo_position.text = MediaPlayerTimeUtil.calculateTime(
                            mMediaPlayer!!.currentPosition / 1000
                        )
                    }

                }
            }

        })

        mMediaPlayer?.setOnCompletionListener {
            isPlaying = if (it.isPlaying) {
                exo_play.setImageResource(R.drawable.icon_pause_play)
                true
            } else {
                exo_play.setImageResource(R.drawable.icon_pause_meditation)
                false
            }
        }

    }

    private fun initMediaPlayer(path: String) {
        mMediaPlayer = MediaPlayerManager.getInstance(this).mediaPlayer
        MediaPlayerManager.getInstance(this).path = path
        val currentPosition = mMediaPlayer?.currentPosition
        duration = mMediaPlayer?.duration ?: 0
        if (currentPosition != null) {
            exo_position.text = MediaPlayerTimeUtil.calculateTime(currentPosition / 1000)
        }
        if (duration != null) {
            exo_duration.text = MediaPlayerTimeUtil.calculateTime(duration / 1000)
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == RESULT_OK && request_code == requestCode) {
            val minute = data?.extras?.getInt("TIME_OFF_MINUTE", 0)
            if (minute != null && minute > 0) {
                if (meditationType == 0) {
                    tv_time_off_pure_music.visibility = View.VISIBLE
                    tv_time_off.visibility = View.GONE
                    mDisposable?.dispose()

                    initRxTimeOff((minute*60*1000).toLong(),0)
                } else {
                    tv_time_off_pure_music.visibility = View.GONE
                    tv_time_off.visibility = View.VISIBLE
                    mDisposable?.dispose()

                    initRxTimeOff((minute*60*1000).toLong(),1)
                }
            } else {
                mDisposable?.dispose()
                tv_time_off.visibility = View.GONE
                tv_time_off_pure_music.visibility = View.GONE
            }
        }
    }


    private fun initRxTimeOff(time:Long,meditationType:Int){

        mObservable = Observable.interval(0,1,TimeUnit.SECONDS)
            .take(time / 1000 + 1)
            .map { t -> time - t * 1000 }
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        mObserver = object : Observer<Long>{
            override fun onSubscribe(d: Disposable) {
                mDisposable = d
            }

            override fun onNext(t: Long) {
                if (meditationType == 0){
                    tv_time_off_pure_music.text = MediaPlayerTimeUtil.formatTimeOff(t)
                }else{
                    tv_time_off.text = MediaPlayerTimeUtil.formatTimeOff(t)
                }
            }

            override fun onError(e: Throwable) {

            }

            override fun onComplete() {
                ToastHelper.show("倒计时结束，停止播放吧！")
                if (mMediaPlayer?.isPlaying == true){
                    mMediaPlayer?.stop()
                    exo_play.setImageResource(R.drawable.icon_pause_meditation)
                }

                if (meditationType == 0){
                    tv_time_off_pure_music.text = ""
                }else{
                    tv_time_off.text = ""
                }

            }

        }

        if (mObserver!=null && mObserver is Observer<Long>){
            mObservable?.subscribe(mObserver as Observer<Long>)
        }

        if (mDisposable?.isDisposed == true && mObserver!=null && mObserver is Observer<Long>){
            mObservable?.subscribe(mObserver as Observer<Long>)
        }

    }


    override fun onDestroy() {
        super.onDestroy()
        mDisposable?.dispose()
        mTimer.cancel()
        MediaPlayerManager.getInstance(this).stop()
        MediaPlayerManager.getInstance(this).release()
    }

    companion object {
        const val request_code = 0x0001
    }



}