package com.yidianling.muse.helper;

import android.content.Context;
import android.media.AudioManager;
import android.media.MediaPlayer;

import java.io.IOException;

public class MediaPlayerManager {

    private Context mContext;
    private MediaPlayer mMediaPlayer;

    private String mPath;

    private volatile static MediaPlayerManager instance;

    private MediaPlayerManager(Context context){
        mContext = context;
        mMediaPlayer = new MediaPlayer();
        mMediaPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);
    }

    public static MediaPlayerManager getInstance(Context context){
        if (instance == null){
            synchronized (MediaPlayerManager.class){
                if (instance == null){
                    instance = new MediaPlayerManager(context);
                }
            }
        }
        return instance;
    }

    public MediaPlayer getMediaPlayer(){
        return mMediaPlayer;
    }

    public void setPath(String path) {
        mPath = path;
        try {
            if (mMediaPlayer.isPlaying()){
                mMediaPlayer.reset();
            }
            mMediaPlayer.setDataSource(path);
            mMediaPlayer.prepare();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void setNewPath(String path){
        mPath = path;
        try {
            mMediaPlayer.reset();
            mMediaPlayer.setDataSource(path);
            mMediaPlayer.prepare();
            mMediaPlayer.start();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String getPath(){
        return mPath;
    }

    public void play(){
        if (mMediaPlayer.isPlaying()) return;
        mMediaPlayer.start();
    }

    public void pause(){
        if (mMediaPlayer.isPlaying()) mMediaPlayer.pause();
    }

    public void seekTo(int position){
        mMediaPlayer.seekTo(position);
    }

    public void stop(){
        if (mMediaPlayer.isPlaying()){
            mMediaPlayer.reset();
        }
    }

    public void release(){
        if (mMediaPlayer!=null) {
            mMediaPlayer.reset();
        }
    }

}
